use 5.005;
use ExtUtils::MakeMaker;
use POSIX;
# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.

#
# $Id: Makefile.PL 413 2006-07-08 05:15:52Z matt $
#

$|++;

my $os   = $^O;

my $cgi    = set_cgi_dir();
my $snmp   = set_snmp_dir();
my $web    = set_web_dir();
my $htdocs = set_htdocs_dir();

sub MY::postamble 
{ 

'CGIBINDIR	= '. $cgi   . '
SNMPDIR	    = '. $snmp  . '
WEBDIR      = '. $web   . '
HTDOCS      = '. $htdocs  . '
TNPIHTML	= /Users/Shared/Sites/www.tnpi.biz

docs:
	pod2text rrdutil.pl      > README

	pod2text rrdutil.pl              > doc/README
	pod2text contrib/apastat.pl      > doc/README.apache
	pod2text contrib/mbmonstatus.pl  > doc/README.mbmon
	pod2text contrib/mysqlstatus.pl  > doc/README.mysql

	pod2html --title="RRDutil README"  rrdutil.pl       > doc/html/README.html

	pod2html --title="Apache Status" contrib/apastat.pl     > doc/html/Apache.html
	pod2html --title="MySQL Status " contrib/mysqlstatus.pl > doc/html/MySQL.html
	pod2html --title="Mbmon Status " contrib/mbmonstatus.pl > doc/html/Mbmon.html
	rm pod2htm*

conf:
	cp rrdutil.conf-dist /usr/local/etc
	install -b contrib/mysqlstatus.pl /usr/local/sbin/
	install -b contrib/mbmonstatus.pl /usr/local/sbin/
	install -b contrib/apastat.pl     /usr/local/sbin/

newconf:
	install rrdutil.conf-dist /usr/local/etc/rrdutil.conf
	install -b contrib/mysqlstatus.pl /usr/local/sbin/
	install -b contrib/mbmonstatus.pl /usr/local/sbin/
	install -b contrib/apastat.pl     /usr/local/sbin/
	install -d /usr/local/rrdutil/data
	install -d -o www -g www $(WEBDIR)/rrdutil/html
	ln -s $(WEBDIR)/rrdutil/html $(HTDOCS)/rrdutil

cgi:
	cp rrdutil.pl $(CGIBINDIR)/rrdutil.cgi

snmp:
	cp contrib/snmpd.conf $(SNMPDIR)/snmpd.conf

freebsd:
	echo "deprecated! Use [make cgi] instead."
	cp rrdutil.pl /usr/local/www/cgi-bin/rrdutil.cgi

darwin:
	echo "deprecated! Use [make cgi] instead."
	cp rrdutil.pl /Library/WebServer/CGI-Executables/rrdutil.cgi

web:
	cp doc/html/README.html      $(TNPIHTML)/internet/manage/rrdutil/README.html
	cp rrdutil.pl                $(TNPIHTML)/internet/manage/rrdutil/rrdutil
	cp rrdutil.conf-dist         $(TNPIHTML)/internet/manage/rrdutil/
	cp RRDutil-$(VERSION).tar.gz $(TNPIHTML)/internet/manage/rrdutil/src
	mv RRDutil-$(VERSION).tar.gz $(TNPIHTML)/internet/manage/rrdutil/RRDutil.tar.gz
';

}

my %hash = (
	'NAME'           => 'RRDutil',
	'VERSION_FROM'   => 'rrdutil.pl', # finds $VERSION
	'EXE_FILES'      => [ 'rrdutil.pl' ],
	'PREREQ_PM'      => { 'Date::Format' => 0, 'RRDs' => 0, 'Net::SNMP' => 0, },
	);


if ( $ExtUtils::MakeMaker::VERSION >= 5.43 ) 
{
	$hash{'AUTHOR'}   = 'Matt Simerson <matt@tnpi.biz>';
	$hash{'ABSTRACT'} = 'A statistic collection, storage, and graph generation utility.';
};

ExtUtils::MakeMaker::WriteMakefile(%hash);

sub WarnNoCgiDir
{
	my ($dir) = @_;

	warn '\n
  WARN: I could not find your cgi-bin directory. It is set for 
        '. $dir .' but that directory does not exist! You will
        need to manually copy rrdutil.pl to there.\n';

	sleep 5;
}

sub set_cgi_dir
{
	my $cgi = $ENV{'CGIBINDIR'};

	if ($cgi && -d $cgi) { return $cgi };


	if    ( $os eq "freebsd" ) { $cgi = "/usr/local/www/cgi-bin"; }
	elsif ( $os eq "darwin" )  { $cgi = "/Library/WebServer/CGI-Executables"; }
	else
	{
		print "trying to find cgi-bin on $os\n";
		if    ( -d "/usr/local/apache/cgi-bin" ) { $cgi = "/usr/local/apache/cgi-bin"; } 
		elsif ( -d "/etc/httpd/cgi-bin"        ) { $cgi = "/etc/httpd/cgi-bin";        }
		elsif ( -d "/var/www/cgi-bin"          ) { $cgi = "/var/www/cgi-bin";          };
	};

	WarnNoCgiDir($cgi) unless ( -d $cgi );

	return $cgi;
};

sub set_snmp_dir
{
	if ( $os eq "darwin" ) { return "/usr/share/snmp"        }
	else                   { return "/usr/local/share/snmp"  };
};

sub set_web_dir
{
	if ( $os eq "darwin" ) { return "/Library/WebServer"        }
	else                   { return "/usr/local"  };
};

sub set_htdocs_dir
{
	if    ( $os eq "darwin" ) { return "/Library/WebServer/Documents" }
	elsif ( $os eq "linux"  ) { return "/var/www/html"                }
	elsif ( -d "/usr/local/www/data" ) { return "/usr/local/www/data" }
	elsif ( -d "/usr/local/www/toaster" ) { return "/usr/local/www/toaster" }
	elsif ( -d "/usr/local/www/apache22/data" ) { return "/usr/local/www/data" }
    else  { 
        print "Odd, I couldn't find your http docroot!\n";
        return "/usr/local/www/toaster"; 
    };
};
