#!/usr/bin/perl
eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
    if 0; # not running under some shell
use strict;

use vars qw( $VERSION );

$VERSION = "1.0";

=head1 NAME

apastat.pl - Outputs current status of apache to a readable format for rrdtool

=head1 SYNOPSIS

- Modified so that it uses perl more extensively instead of running lynx. 
  Also modified so that it uses a wildcard for Busy and Idle servers instead 
  of having to hit the server twice, we can do it all in one swell swoop. 
  (Patrick Bulteel)
 
- Changed the output so snmp can use it (Rob Lensen)

=head1 DEPENDENCIES

Make sure you have the following in your httpd.conf:

 LoadModule status_module      libexec/apache/mod_status.so
 AddModule mod_status.c (only for Apache 1.X)
 ExtendedStatus On

 <Location /server-status>
    SetHandler server-status
    Order deny,allow
    Deny from all
    Allow from localhost
 </Location>

=cut

use LWP::UserAgent; 
use HTTP::Request; 
use HTTP::Response; 
use Getopt::Std;
use vars qw( $opt_r $opt_v);
getopts('rv');

CheckOption();

sub printServerStatus($)
{
	my ($hostname) = @_; 
	my $hostname = shift;
	my $ua = new LWP::UserAgent; 
	# I added this to the original O'reilly code... 
	my $fullurl = "http://" .$hostname."/server-status?auto"; 
 
	# I changed this from the original O'reilly code... 
	my $request = new HTTP::Request('GET', $fullurl); 
	my $response = $ua->request($request); 
		
	if ($response->is_success) 
	{ 	
		# I grabbed this from the original unix_apachestats.pl 
		$response->content =~ /Total Accesses: (.*)/; 
		print "apache_total_hits:".$1.":"; 
		$response->content =~ /Total kBytes: (.*)/; 
		print "apache_total_kbytes:".$1.":"; 
		# I wildcarded the Busy and Idle stats... =) 
		$response->content =~ /Busy.*: (.*)/; 
		print "apache_busy_workers:".$1.":"; 
		$response->content =~ /Idle.*: (.*)/; 
		print "apache_idle_workers:".$1.":"; 	
		$response->content =~ /ReqPerSec.*: (.*)/;
		print "req_per_sec:".$1.":";
		#$response->content =~ /BytesPerSec.*: (.*)/;
		#print "bytes_per_sec:".$1.":";
		$response->content =~ /BytesPerReq.*: (.*)/;
		print "bytes_per_req:".$1."\n";
	}
	else
	{ 
		print "apache_total_hits:0:"; 
		print "apache_total_kbytes:0:"; 
		print "apache_busy_workers:0:"; 
		print "apache_idle_workers:0:";
		print "req_per_sec:0:";
		print "bytes_per_sec:0:";
		print "bytes_per_req:0\n";	
	}	 
	
	
} 
sub CheckOption()
{
	my $option = $ARGV[0];
	if ($option)
	{
		printServerStatus($option);
	}
	else
        {
                print "\n Need an argument like this:\n\n";
                die "apache_status.pl hostname\n\n";
        }
}

=head1 AUTHORS

  Taken from ftp://ftp.ora.com/published/oreilly/nutshell/web-client/ 
  Modified by: Patrick Bulteel 
  And : Rob Lensen

=head1 SEE ALSO

  http://www.tnpi.biz/internet/manage/rrdutil/

=cut
