#!/usr/bin/perl
use strict;

use vars qw( $VERSION );

$VERSION = "1.1";

=head1 NAME

mbmonstatus

=head1 SYNOPSIS

outputs current temperature,voltage and fanspeed readable for snmp

=cut

use Getopt::Std;
use vars qw( $opt_r $opt_v);
getopts('rv');

CheckOption();

sub getStatus
{
	my %result; 
	my ($key, $val ) ; 
	open( FD, "/usr/local/bin/mbmon -rc1|" )  || die "ERROR: Cannot run mbmon\n" ; 
	
	while( $_ = <FD> ) 
	{ 
		next    unless( /^([A-Za-z][^:\s]+)\s*:\s*([+\-]{0,1}[\d\.]+)/ ) ; 
		$key    = $1 ; 
		$val    = $2 ; 
		$key    =~ y/A-Z/a-z/ ; 
		$result{$key} = $val ; 
#		print( "\$result{$key} = \"$val\"\n")  ; 
	} 
	close( FD) ; 
	return \%result;
}

sub printServerStatus($$)
{
	my ($type, $status) = @_; 
	my $type = shift;
	my $status = shift;	
	if( $type eq "temperature")
	{
		if (defined($status))
		{ 
			print "mbmon_temp0:".$status->{temp0}.":mbmon_temp1:".$status->{temp1}.":mbmon_temp2:".$status->{temp2}."\n";
		}
		else
		{ 
			print "mbmon_temp0:0:mbmon_temp1:0:mbmon_temp2:0\n";
		}	 
	}
	elsif( $type eq "voltage" )
	{
		if (defined($status))
		{ 
			print "mbmon_vc0:".$status->{vc0}.":mbmon_vc1:".$status->{vc1}. 
				":mbmon_v33:".$status->{v33}.":mbmon_v50p:".$status->{v50p}. 
				":mbmon_v12p:".$status->{v12p}.":mbmon_v12n:".$status->{v12n}.
				":mbmon_v50n:".$status->{v50n}."\n";
		}
		else 
		{
			print "mbmon_vc0:0:mbmon_vc1:0:mbmon_v33:0:mbmon_v50p:0:mbmon_v12p:0:mbmon_v12n:0:mbmon_v50n:0\n";
		}		
	}
	elsif ($type eq "fanspeed" )
	{
		if (defined($status))
		{
			print "mbmon_fan0:".$status->{fan0}.":mbmon_fan1:".$status->{fan1}.":mbmon_fan2:".$status->{fan2}."\n";
		}
		else
		{
			print "mbmon_fan0:0:mbmon_fan1:0:mbmon_fan2:0\n";
		}
	}
} 

sub CheckOption()
{
	my $option = $ARGV[0];
	if ($option)
	{
		if ( $option eq "temp") 
		{
			my $status = getStatus();
			my $soort = "temperature";
			printServerStatus($soort, $status);
		}
		elsif ( $option eq "volt")
		{
			my $status = getStatus();
			my $soort = "voltage";
			printServerStatus($soort, $status);
		}
		elsif ( $option eq "fan")
		{
			my $status = getStatus();
			my $soort = "fanspeed";
			printServerStatus($soort, $status);
		}
	}
	else
	{
		die "\n Need an argument like this:\n\n $0 [temp | volt | fan ]\n\n";
	}
}

=head1 DEPENDENCIES

 mbmon - /usr/ports/sysutils/xmbmon

=head1 AUTHOR

 Rob Lensen <rob@bsdfreaks.nl>

=head1 CHANGELOG

 v1.1 - added additional documentation (Matt)
      - noted xmbmon port dependency

 v1.0 - authored by Rob Lensen

=head1 BUGS

None known. Report any to author.

=head1 TODO


=head1 SEE ALSO

http://www.tnpi.biz/internet/manage/rrdutil/

=head1 COPYRIGHT

Copyright (c) 2003-2004, The Network People, Inc.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  

=cut
