#!/usr/bin/perl
use strict;

#
# $Id: Graphs.pm 413 2006-07-08 05:15:52Z matt $
#

package RRDutil::Graphs;

use Carp;
use POSIX qw(uname);
use lib "/opt/local/lib/perl/";  # for DarwinPorts compat
use RRDs;
use lib "../";
use RRDutil;          my $rrdutil = RRDutil->new();
use RRDutil::Utility; my $utility = RRDutil::Utility->new();
use vars qw($VERSION);

$VERSION = '1.5';

=head1 NAME

RRDutil::Graphs - Perl module for rrdutil.pl


=head1 SYNOPSIS

RRDutil::Graphs


=head1 DESCRIPTION

=cut


sub new
{
	my ($class, $name) = @_;
	my $self = { name => $name };
	bless ($self, $class);
	return $self;
};


##
# Graph Creation
##
# -----------------------------------------------------------------------------

# DEF:vname=rrd:ds-name:CF
# CDEF:vname=rpn-expression
# PRINT:vname:CF:format
# GPRINT:vname:CF:format
# COMMENT: text
# HRULE:value#rrggbb[:legend]
# VRULE:time#rrggbb[:legend]
# LINE{1|2|3}:vname[#rrggbb[:legend]]
# AREA:vname[#rrggbb[:legend]]
# STACK:vname[#rrggbb[:legend]]
# format: %le, %lg, %lf  = num precision
# format: %s = scale, 
# format: \j \l \c \r (justify)  \g pack spaces
# format: \s (vertical space) - \n newline

sub font_set
{
	my ($conf) = @_;

	if ($conf->{'rrdtool_version'} && $conf->{'rrdtool_version'} == 2) {
		return "--font=LEGEND:7:/usr/local/share/rrdtool/fonts/DejaVuSansMono-Roman.ttf ";
	};
	return 0;
}

sub format_colors 
{
	foreach (@_) { 
		if    ( $_ eq "black")        { $_ = '000000' }
		elsif ( $_ eq "violet")       { $_ = '9900FF' }
		elsif ( $_ eq 'blue')         { $_ = '0000FF' }
		elsif ( $_ eq "light-blue")   { $_ = '3399FF' }
		elsif ( $_ eq "aqua")         { $_ = '00FFFF' }
		elsif ( $_ eq "green")        { $_ = '00CC00' }
		elsif ( $_ eq "light-green")  { $_ = '99FF00' }
		elsif ( $_ eq "yellow")       { $_ = 'FFFF00' }
		elsif ( $_ eq "light-orange") { $_ = 'FFCC00' }
		elsif ( $_ eq "orange")       { $_ = 'FF6600' }
		elsif ( $_ eq "red")          { $_ = 'FF0000' }
		else {               
			unless ($_ =~ /^[A-F0-9]{6}$/ ) { 
				print "color $_ unknown, substituting black.\n<br>";
				$_ = '000000';
			};
		};
	};
	return @_;
};

sub formatted_timestamp
{
	my ($conf) = @_;

	my $ts = Date::Format::time2str("%a %b %e %T %Z %Y", time);
	if ($conf->{'rrdtool_version'} == 2 ) {
		$ts =~ s/:/\\:/g;  # escape : characters with a \
		print "ts: $ts\n<br>" if $conf->{'debug'};
	};
	return $ts;
}

sub generate($$$;$)
{
	my ($self, $hosts, $conf, $monitors, $types_selected) = @_;

	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};

	print "generate: entering sub<br></br>" if $debug;

	foreach my $host ( @$hosts )
	{
		print "generate: iterating over each host ($host)<br></br>" if $debug;
		
		next unless ( $rrdutil->check_datadir( $host, "read", $conf ) );
		next unless ( $rrdutil->check_html_dir( $htmldir, $host, "read" ) );

		my $polls = $monitors->{$host};     # list of all polls for that host

		my $i = 0;
		my $k = 0;
		
		POLL: foreach my $poll (@$polls)    # Iterate through list of all polls
		{
			my $doit = 0;
					
			if ( $types_selected)           # Only generate graphs user selected
			{
				my ($type, $detail);
				until ( $type ) { ($type, $detail) = each %$poll };

				print "Generate Graphs: while type: $type  detail: $detail->[0] $detail->[1]<br></br>" if $debug;

				$doit = 1 if ( $types_selected->{$type} );
				next POLL unless ($doit);

				my $mon = $detail->[0];
				my $descr = $detail->[1];

				print "Generate Graphs doit: type: $type  mon: $mon descr: $descr <br></br>" if $debug;

				if    ( $type eq "storage"  && $i == 0 ) 
				{
					$i++;  GraphStorage ($host, $polls, $conf);
				} 
				elsif ( $type eq "www" and $k == 0) 
				{
					$k++;  GraphWebHits ($host, $polls, $conf);
				} 
				elsif ($type eq "load" or $type eq "cpu")
				{ 
					GraphLoad ($host, $descr, $conf);
					GraphCPU  ($host, $descr, $conf);
				}
				elsif ($type eq "disk"   ) { GraphDiskBytes    ($host, $mon, $descr, $conf) }
				elsif ($type eq "memory" ) { GraphMemory       ($host, $descr, $conf) }
				elsif ($type eq "mail"   ) { GraphMail         ($host, $mon, $descr, $conf) }
				elsif ($type eq "mysql"  ) { GraphMysql        ($host, $mon, $descr, $conf) }
				elsif ($type eq "net"    ) { GraphNetwork      ($host, $descr, $mon, $conf) }
				elsif ($type eq "mbmon"  ) { GraphMbmon	       ($host, $mon, $descr, $conf) }
				elsif ($type eq "apache" ) { GraphApache       ($host, $mon, $descr, $conf) }
				elsif ($type eq "pf"     ) { GraphPF           ($host, $mon, $descr, $conf) };
			}
			else                             # Generate all graphs
			{
				my ($type, $detail);
				until ( $type ) { ($type, $detail) = each %$poll };

				my $mon = $detail->[0];
				my $descr = $detail->[1];

				if    ( $type eq "storage"  && $i == 0 ) 
				{
					$i++;  GraphStorage ($host, $polls, $conf);
				} 
				elsif ( $type eq "www"  && $k == 0) 
				{
					$k++;  GraphWebHits ($host, $polls, $conf);
				} 
				elsif ($type eq "load" or $type eq "cpu")
				{
					GraphLoad ($host, $descr, $conf);
					GraphCPU  ($host, $descr, $conf);
				}
				elsif ($type eq "disk"   ) { GraphDiskBytes    ($host, $mon, $descr, $conf) }
				elsif ($type eq "memory" ) { GraphMemory       ($host, $descr, $conf) }
				elsif ($type eq "mail"   ) { GraphMail         ($host, $mon, $descr, $conf) }
				elsif ($type eq "mysql"  ) { GraphMysql        ($host, $mon, $descr, $conf) }
				elsif ($type eq "net"    ) { GraphNetwork      ($host, $descr, $mon, $conf) }
				elsif ($type eq "mbmon"  ) { GraphMbmon        ($host, $mon, $descr, $conf) }
				elsif ($type eq "apache" ) { GraphApache       ($host, $mon, $descr, $conf) }
				elsif ($type eq "pf"     ) { GraphPF           ($host, $mon, $descr, $conf) };
			};
		};
	};
}


sub GetLegend($$;$$$$)
{
	my ($measure, $conf, $nums, $precision, $nolabel, $s) = @_;
	my ($last, $avg, $min, $max, @rrdcmd);

	my $debug = $conf->{'debug'};

	$nums ||= 6;

	unless ($s) { $s = "" } else { $s = "%s" };

	if ($nolabel) { $last = "" } else { $last = "Current\\:" };
	if ($nolabel) { $avg  = "" } else { $avg  = "Avg\\:"     };
	if ($nolabel) { $min  = "" } else { $min  = "Min\\:"     };
	if ($nolabel) { $max  = "" } else { $max  = "Max\\:"     };

	push @rrdcmd, "GPRINT:$measure:LAST:$last %${nums}.${precision}lf $s";
	push @rrdcmd, "GPRINT:$measure:AVERAGE:$avg %${nums}.${precision}lf $s";
	push @rrdcmd, "GPRINT:$measure:MIN:$min %${nums}.${precision}lf $s";
	push @rrdcmd, "GPRINT:$measure:MAX:$max %${nums}.${precision}lf $s";

	print "GetLegend: returning legend for $measure.\n" if $debug;
	return @rrdcmd;
};

sub GenerateGraph($$@)
{
	my ($conf, $image, @rrdcmd) = @_;

	my $debug = $conf->{'debug'};

	print "GenerateGraph: rrdcmd: \n\t" . join("\n\t", @rrdcmd) . "<br>\n" if $debug; 
	print "GenerateGraph: building $image..." if $debug;

	RRDs::graph ( @rrdcmd );
	
	if (RRDs::error) 
	{ 
		print "ERROR generating $image: ". RRDs::error . "\n";
		carp "ERROR generating $image: ". RRDs::error . "\n";
	}
	else
	{
		print "success\n<br>" if $debug;
	};

#	return ( CheckRRDsError($debug) );
};


sub GraphMail($$$$)
{
	my ($host, $mon, $descr, $conf) = @_;
	my @rrdcmd;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	print "GraphMail being called for $host ($mon, $descr)<br>\n" if $debug;

	my $datasrc = "$datadir/$host/${descr}.rrd";
	$utility->file_check_readable($datasrc);

	my @color = format_colors( split(',', $conf->{'colors'}) );

	if ( $mon eq "send" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMail: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Deliveries / Hour";
		push @rrdcmd, "--title=Mail Deliveries";
		push @rrdcmd, "--height=150";
		push @rrdcmd, "--width=480";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:delivery_success:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:delivery_deferral:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:delivery_failure:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:message_bytes:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:message_new:AVERAGE";
		push @rrdcmd, "DEF:a6=$datasrc:message_bounce:AVERAGE";
		push @rrdcmd, "DEF:a7=$datasrc:message_end:AVERAGE";
		push @rrdcmd, "DEF:a8=$datasrc:concurrencyremote:AVERAGE";

		push @rrdcmd, "CDEF:success=a1,3600,\*";
		push @rrdcmd, "CDEF:deferral=a2,3600,\*";
		push @rrdcmd, "CDEF:failure=a3,3600,\*";

		push @rrdcmd, "COMMENT:                Min     Cur     Avg     Max        ";
		push @rrdcmd, "GPRINT:a8:AVERAGE:Remote Busy\\: %3.0lf\\n";

		push @rrdcmd, "AREA:success#00FF00: success ";
		push @rrdcmd, "GPRINT:success:MIN: %5.0lf";
		push @rrdcmd, "GPRINT:success:LAST: %5.0lf";
		push @rrdcmd, "GPRINT:success:AVERAGE: %5.0lf";
		push @rrdcmd, "GPRINT:success:MAX: %5.0lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:deferral#FFFF00: deferral";
		push @rrdcmd, "GPRINT:deferral:MIN: %5.0lf";
		push @rrdcmd, "GPRINT:deferral:LAST: %5.0lf";
		push @rrdcmd, "GPRINT:deferral:AVERAGE: %5.0lf";
		push @rrdcmd, "GPRINT:deferral:MAX: %5.0lf";
		push @rrdcmd, "COMMENT:        Last updated\\n";

		push @rrdcmd, "STACK:failure#FF0000: failure ";
		push @rrdcmd, "GPRINT:failure:MIN: %5.0lf";
		push @rrdcmd, "GPRINT:failure:MAX: %5.0lf";
		push @rrdcmd, "GPRINT:failure:LAST: %5.0lf";
		push @rrdcmd, "GPRINT:failure:AVERAGE: %5.0lf";
		push @rrdcmd, "COMMENT:        " . formatted_timestamp($conf) . "\\n";
		push @rrdcmd, "COMMENT:\\s";

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif ($mon eq "rbl" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMail: passed checks.<br>\n" if $debug;

	         @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Blocks / Hour";
		push @rrdcmd, "--title=RBL Spam Blocks";
		#push @rrdcmd, "--height=200";
		push @rrdcmd, "--width=480";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:smtp1:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:smtp2:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:smtp3:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:smtp4:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:smtp5:AVERAGE";
		push @rrdcmd, "DEF:a6=$datasrc:smtp6:AVERAGE";
		push @rrdcmd, "DEF:a7=$datasrc:smtp7:AVERAGE";
		push @rrdcmd, "DEF:a8=$datasrc:smtp8:AVERAGE";
		push @rrdcmd, "CDEF:spamhaus=a1,3600,\*";
		push @rrdcmd, "CDEF:spamcop=a2,3600,\*";
		push @rrdcmd, "CDEF:dsbl=a3,3600,\*";
		push @rrdcmd, "CDEF:ordb=a4,3600,\*";
		push @rrdcmd, "CDEF:maps=a5,3600,\*";
		push @rrdcmd, "CDEF:sorbs=a6,3600,\*";
		push @rrdcmd, "CDEF:ignorant=a7,3600,\*";
		push @rrdcmd, "CDEF:services=a8,3600,\*";

		push @rrdcmd, "COMMENT:                Min    Cur    Avg    Max                        Min    Cur    Avg    Max\\n";

		push @rrdcmd, "AREA:maps#$color[0]:mail-abuse";
		push @rrdcmd, "GPRINT:maps:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:maps:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:maps:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:maps:MAX: %4.0lf    ";
		#push @rrdcmd, "COMMENT:\\n";
	
		push @rrdcmd, "STACK:ignorant#$color[1]:rfc-ignorant";
		push @rrdcmd, "GPRINT:ignorant:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:ignorant:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:ignorant:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:ignorant:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:ordb#$color[2]:ordb.org  ";
		push @rrdcmd, "GPRINT:ordb:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:ordb:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:ordb:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:ordb:MAX: %4.0lf    ";
		#push @rrdcmd, "COMMENT:\\n";
	
		push @rrdcmd, "STACK:dsbl#$color[3]:dsbl.org    ";
		push @rrdcmd, "GPRINT:dsbl:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:dsbl:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:dsbl:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:dsbl:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:spamcop#$color[4]:spamcop   ";
		push @rrdcmd, "GPRINT:spamcop:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:spamcop:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:spamcop:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:spamcop:MAX: %4.0lf    ";
		#push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:services#$color[5]:services.net";
		push @rrdcmd, "GPRINT:services:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:services:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:services:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:services:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:sorbs#$color[6]:sorbs     ";
		push @rrdcmd, "GPRINT:sorbs:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:sorbs:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:sorbs:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:sorbs:MAX: %4.0lf    ";
		#push @rrdcmd, "COMMENT:\\n";
	
		push @rrdcmd, "STACK:spamhaus#$color[7]:spamhaus    ";
		push @rrdcmd, "GPRINT:spamhaus:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:spamhaus:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:spamhaus:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:spamhaus:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		GenerateGraph($conf, $image, @rrdcmd);

		# second graph

		$image   = "$htmldir/$host/${descr}-2.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMail: passed checks.<br>\n" if $debug;

		my $style = "STACK";  # LINE1, LINE2, STACK

	         @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Blocks / Hour";
		push @rrdcmd, "--title=Invalid Message Blocks";
		#push @rrdcmd, "--height=200";
		push @rrdcmd, "--width=480";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:smtp13:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:smtp14:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:smtp15:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:smtp16:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:smtp17:AVERAGE";
		push @rrdcmd, "DEF:a6=$datasrc:smtp18:AVERAGE";
		push @rrdcmd, "DEF:a7=$datasrc:smtp19:AVERAGE";
		push @rrdcmd, "DEF:a8=$datasrc:smtp20:AVERAGE";
		push @rrdcmd, "CDEF:badhelo=a1,3600,\*";
		push @rrdcmd, "CDEF:badfrom=a2,3600,\*";
		push @rrdcmd, "CDEF:badto=a3,3600,\*";
		push @rrdcmd, "CDEF:dns=a4,3600,\*";
		push @rrdcmd, "CDEF:other=a5,3600,\*";
		push @rrdcmd, "CDEF:chkuser=a6,3600,\*";
		push @rrdcmd, "CDEF:spf=a7,3600,\*";
		push @rrdcmd, "CDEF:virus=a8,3600,\*";

		push @rrdcmd, "COMMENT:               Min    Cur    Avg    Max                         Min    Cur    Avg    Max\\n";

		$style eq "STACK" ?  push @rrdcmd, "AREA:badhelo#$color[0]: badhelo " : push @rrdcmd, "LINE2:badhelo#$color[0]: badhelo ";
		push @rrdcmd, "GPRINT:badhelo:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:badhelo:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:badhelo:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:badhelo:MAX: %4.0lf        ";

		push @rrdcmd, "$style:dns#$color[1]: dns     ";
		push @rrdcmd, "GPRINT:dns:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:dns:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:dns:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:dns:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";
	
		push @rrdcmd, "$style:badfrom#$color[2]: badfrom ";
		push @rrdcmd, "GPRINT:badfrom:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:badfrom:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:badfrom:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:badfrom:MAX: %4.0lf        ";

		push @rrdcmd, "$style:other#$color[3]: other   ";
		push @rrdcmd, "GPRINT:other:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:other:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:other:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:other:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "$style:badto#$color[4]: badto   ";
		push @rrdcmd, "GPRINT:badto:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:badto:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:badto:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:badto:MAX: %4.0lf        ";

		push @rrdcmd, "$style:spf#$color[5]: spf     ";
		push @rrdcmd, "GPRINT:spf:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:spf:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:spf:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:spf:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "$style:chkuser#$color[6]: chkuser ";
		push @rrdcmd, "GPRINT:chkuser:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:chkuser:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:chkuser:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:chkuser:MAX: %4.0lf        ";

		push @rrdcmd, "$style:virus#$color[7]: virus   ";
		push @rrdcmd, "GPRINT:virus:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:virus:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:virus:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:virus:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\n";

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif ( $mon eq "mua" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

   		@rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Connections / Hour";
		push @rrdcmd, "--title=Mail User Agent Connections";
		#push @rrdcmd, "--height=200";
		push @rrdcmd, "--width=480";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:imap_connect:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:imap_ssl_connect:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:pop3_connect:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:pop3_ssl_connect:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:imap_success:AVERAGE";
		push @rrdcmd, "DEF:a6=$datasrc:imap_ssl_success:AVERAGE";
		push @rrdcmd, "DEF:a7=$datasrc:pop3_success:AVERAGE";
		push @rrdcmd, "DEF:a8=$datasrc:pop3_ssl_success:AVERAGE";
		push @rrdcmd, "DEF:a9=$datasrc:webmail_connect:AVERAGE";
		push @rrdcmd, "DEF:a10=$datasrc:webmail_success:AVERAGE";
		push @rrdcmd, "DEF:a11=$datasrc:smtpauth_connect:AVERAGE";
		push @rrdcmd, "DEF:a12=$datasrc:smtpauth_success:AVERAGE";
	
		push @rrdcmd, "CDEF:imap=a5,3600,\*";
		push @rrdcmd, "CDEF:imapssl=a6,3600,\*";
		push @rrdcmd, "CDEF:pop=a7,3600,\*";
		push @rrdcmd, "CDEF:popssl=a8,3600,\*";
		push @rrdcmd, "CDEF:webmail=a10,3600,\*";
		push @rrdcmd, "CDEF:smtpauth=a12,3600,\*";
	
		push @rrdcmd, "COMMENT:               Min     Cur     Avg     Max                      Min     Cur     Avg     Max\\j";

		my $style = "LINE1";

		push @rrdcmd, "$style:imap#$color[0]: imap    ";
		push @rrdcmd, "GPRINT:imap:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:imap:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:imap:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:imap:MAX: %4.0lf    ";

		push @rrdcmd, "$style:pop#$color[1]: pop     ";
		push @rrdcmd, "GPRINT:pop:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:pop:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:pop:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:pop:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\j";

		push @rrdcmd, "$style:imapssl#$color[2]: imapssl ";
		push @rrdcmd, "GPRINT:imapssl:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:imapssl:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:imapssl:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:imapssl:MAX: %4.0lf    ";

		push @rrdcmd, "$style:popssl#$color[3]: popssl  ";
		push @rrdcmd, "GPRINT:popssl:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:popssl:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:popssl:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:popssl:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\j";

		push @rrdcmd, "$style:webmail#$color[7]: webmail ";
		push @rrdcmd, "GPRINT:webmail:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:webmail:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:webmail:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:webmail:MAX: %4.0lf    ";

		push @rrdcmd, "$style:smtpauth#$color[5]: smtpauth";
		push @rrdcmd, "GPRINT:smtpauth:MIN: %4.0lf";
		push @rrdcmd, "GPRINT:smtpauth:LAST: %4.0lf";
		push @rrdcmd, "GPRINT:smtpauth:AVERAGE: %4.0lf";
		push @rrdcmd, "GPRINT:smtpauth:MAX: %4.0lf";
		push @rrdcmd, "COMMENT:\\j";

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif ( $mon eq "spam" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

   		@rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Messages / Hour";
		push @rrdcmd, "--title=SpamAssassin Mail Filtering";
		#push @rrdcmd, "--height=200";
		push @rrdcmd, "--width=480";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:sa1:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:sa2:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:sa3:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:sa4:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:sa5:AVERAGE";
		push @rrdcmd, "DEF:a6=$datasrc:sa6:AVERAGE";
		push @rrdcmd, "DEF:a7=$datasrc:sa7:AVERAGE";
		push @rrdcmd, "DEF:a8=$datasrc:sa8:AVERAGE";
		push @rrdcmd, "DEF:a9=$datasrc:sa9:AVERAGE";

		push @rrdcmd, "CDEF:spam_count=a1,3600,\*";
		push @rrdcmd, "CDEF:ham_count=a2,3600,\*";
		push @rrdcmd, "CDEF:spam_score=a3,.01,\*";
		push @rrdcmd, "CDEF:ham_score=a4,.01,\*";
		push @rrdcmd, "CDEF:threshhold=a5,.01,\*";
		push @rrdcmd, "CDEF:spam_seconds=a6,.01,\*";
		push @rrdcmd, "CDEF:ham_seconds=a7,.01,\*";
		push @rrdcmd, "CDEF:spam_bytes=a8,.01,\*";
		push @rrdcmd, "CDEF:ham_bytes=a9,.01,\*";

		push @rrdcmd, "COMMENT:             mess/hr     avg score   avg seconds   avg bytes \\n";
		push @rrdcmd, "AREA:ham_count#00FF00: clean   ";
		push @rrdcmd, "GPRINT:ham_count:LAST: %3.2lf      ";
		push @rrdcmd, "GPRINT:ham_score:LAST: %3.2lf      ";
		push @rrdcmd, "GPRINT:ham_seconds:LAST: %3.2lf    ";
		push @rrdcmd, "GPRINT:ham_bytes:LAST: %3.2lf %s ";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:spam_count#FFA500: spam    ";
		push @rrdcmd, "GPRINT:spam_count:LAST: %3.2lf      ";
		push @rrdcmd, "GPRINT:spam_score:LAST: %3.2lf      ";
		push @rrdcmd, "GPRINT:spam_seconds:LAST: %3.2lf    ";
		push @rrdcmd, "GPRINT:spam_bytes:LAST: %3.2lf %s ";

#		push @rrdcmd, "GPRINT:imap:AVERAGE: %4.0lf";

		GenerateGraph($conf, $image, @rrdcmd);
	};
};

sub GraphWebHits($$$)
{
	my ($host, $polls, $conf) = @_;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $image   = "$htmldir/$host/webhits.png";

	return 0 if ( -e $image && ! $utility->file_check_writable($image) );

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my   @rrdGroup = $image;
	push @rrdGroup, "--start=-$seconds";
	push @rrdGroup, "--vertical-label=Hits / Day";
	push @rrdGroup, "--title=Aggregate Web Hits";
	push @rrdGroup, "COMMENT:\\s";
	push @rrdGroup, font_set($conf) if font_set($conf);

	my $j = 0;

	POLL: foreach my $poll (@$polls)          # Iterate through list of all polls
	{
		my ($type, $detail);
		until ( $type ) { ($type, $detail) = each %$poll };
		
		next POLL unless ($type eq "www");
	
		my $mon = $detail->[0];
		my $descr = $detail->[1];

		print "GraphWebHits: ($type, $mon, $descr)\n <br> " if $debug;

		my @color = format_colors( split(',', $conf->{'colors'}) );

		if ($mon eq "group")  
		{
			print "GraphWebHits: Adding Group RRD for $descr. <br>\n" if $debug;

			my $file = "$datadir/$host/${descr}.rrd";
			$utility->file_check_readable($file);

			push @rrdGroup, "DEF:hits${j}=$file:hits:AVERAGE";
			#push @rrdGroup, "LINE1:hits${j}#000000:$descr ";

			if    ($j == 0) { push @rrdGroup, "AREA:hits${j}#$color[$j]: $descr";  }
			else            { push @rrdGroup, "STACK:hits${j}#$color[$j]: $descr"; }

			push @rrdGroup, "GPRINT:hits${j}:MAX:Max\\: %6.0lf";
			push @rrdGroup, "GPRINT:hits${j}:LAST:Current\\: %6.0lf";
			push @rrdGroup, "COMMENT:\\j";

			$j++;
		} 
		elsif ($mon eq "solo") 
		{
			my $simage = "$htmldir/$host/${descr}.png";
			return 0 if ( -e $simage && ! $utility->file_check_writable($simage) );

			#print "Group Solo for descr: $descr. .\n" if $debug;
			#print "Group Solo for image: $simage. .\n" if $debug;

			my $file = "$datadir/$host/${descr}.rrd";
			$utility->file_check_readable($file);

			my   @rrdcmd = $simage;
			push @rrdcmd, "--start=-$seconds";
			push @rrdcmd, "--title=$descr Web Hits";
			push @rrdcmd, "--vertical-label=Hits";
			push @rrdGroup, font_set($conf) if font_set($conf);
			push @rrdcmd, "DEF:hits=$file:hits:AVERAGE";
			push @rrdcmd, "AREA:hits#008000:$descr ";
			push @rrdcmd, "GPRINT:hits:MAX:Max\\: %6.0lf";
			push @rrdcmd, "GPRINT:hits:LAST:Current\\: %6.0lf\\j";
			print	"GraphWebHits: building $simage..." if $debug;

			GenerateGraph($conf, $simage, @rrdcmd);
		} 
		else 
		{
			carp "What is $mon?  skipping.\n";
		};
	};

	GenerateGraph($conf, $image, @rrdGroup);
};

sub GraphCPU($$$)
{
	my ($host, $descr, $conf) = @_;

	my $days    = $conf->{'days'}   || 1;
	my $debug   = $conf->{'debug'}  || 0;
	my $htmldir = $conf->{'htmldir'} || "/usr/local/rrdutil/html";
	my $datadir = $conf->{'datadir'} || "/usr/local/rrdutil/data";

	my $datasrc = "$datadir/$host/${descr}.rrd";
	my $image   = "$htmldir/$host/${descr}_detail.png";

	if ( -e $image && ! $utility->file_check_writable($image) ) {
		print "ERROR: file permissions don't allow me to update $image!\n";
		return 0 
	};
	$utility->file_check_readable($datasrc);

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my   @rrdcmd = $image;
	push @rrdcmd, "--start=-$seconds";
	push @rrdcmd, "--lower-limit=0";
	push @rrdcmd, "--upper-limit=100";
	push @rrdcmd, "--vertical-label=Percentage";
	push @rrdcmd, "--title=CPU Activity";
	#push @rrdcmd, font_set($conf) if font_set($conf);

	push @rrdcmd, "DEF:user=$datasrc:user:AVERAGE";
	push @rrdcmd, "DEF:system=$datasrc:system:AVERAGE";
#	push @rrdcmd, "DEF:idle=$datasrc:idle:AVERAGE";

	push @rrdcmd, "AREA:user#000000:user  ";
	push @rrdcmd, "GPRINT:user:MIN:Min\\: %3.1lf";
	push @rrdcmd, "GPRINT:user:MAX:Max\\: %3.1lf";
	push @rrdcmd, "GPRINT:user:LAST:Current\\: %3.1lf";
	push @rrdcmd, "GPRINT:user:AVERAGE:Average\\: %3.1lf";
	push @rrdcmd, "COMMENT:\\n";

	push @rrdcmd, "STACK:system#FF0000:system";
	push @rrdcmd, "GPRINT:system:MIN:Min\\: %3.1lf";
	push @rrdcmd, "GPRINT:system:MAX:Max\\: %3.1lf";
	push @rrdcmd, "GPRINT:system:LAST:Current\\: %3.1lf";
	push @rrdcmd, "GPRINT:system:AVERAGE:Average\\: %3.1lf";

	GenerateGraph($conf, $image, @rrdcmd);
};

sub GraphLoad($$$)
{
	my ($host, $descr, $conf) = @_;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $datasrc = "$datadir/$host/${descr}.rrd";
	my $image   = "$htmldir/$host/${descr}.png";

	return 0 if ( -e $image && ! $utility->file_check_writable($image) );
	$utility->file_check_readable($datasrc);

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my   @rrdcmd = $image;
	push @rrdcmd, "--start=-$seconds";

	push @rrdcmd, "-v Load Average";
	push @rrdcmd, "--lower-limit=0";
	push @rrdcmd, "--upper-limit=max";
	#push @rrdcmd, "--rigid";
	push @rrdcmd, "--height=200";
	push @rrdcmd, "--width=480";
	push @rrdcmd, "--units-exponent=1";
	push @rrdcmd, font_set($conf) if font_set($conf);

	push @rrdcmd, "DEF:one=$datasrc:one:AVERAGE";
	push @rrdcmd, "DEF:five=$datasrc:five:AVERAGE";
	push @rrdcmd, "DEF:fifteen=$datasrc:fifteen:AVERAGE";
	push @rrdcmd, "DEF:user=$datasrc:user:AVERAGE";
	push @rrdcmd, "DEF:system=$datasrc:system:AVERAGE";

	push @rrdcmd, "CDEF:percent=user,system,\+";
	push @rrdcmd, "CDEF:blue=percent,10,LE,fifteen,0,IF";
	push @rrdcmd, "CDEF:lblue=percent,10,GT,percent,20,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:aqua=percent,20,GT,percent,30,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:green=percent,30,GT,percent,40,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:gyello=percent,40,GT,percent,50,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:yellow=percent,50,GT,percent,60,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:yorang=percent,60,GT,percent,70,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:oyello=percent,70,GT,percent,80,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:orange=percent,80,GT,percent,90,LE,+,2,EQ,fifteen,0,IF";
	push @rrdcmd, "CDEF:red=percent,90,GT,fifteen,0,IF";

	push @rrdcmd, "COMMENT:CPU Utilization(%)\\n";
	push @rrdcmd, "COMMENT: ";
	push @rrdcmd, "AREA:blue#0000FF: 0-10%";
	push @rrdcmd, "AREA:lblue#3399FF:10-20%";
	push @rrdcmd, "AREA:aqua#00FFFF:20-30%";
	push @rrdcmd, "AREA:green#00CC00:30-40%";
	push @rrdcmd, "AREA:gyello#99FF00:40-50%";
	push @rrdcmd, "GPRINT:percent:MIN:     Min\\: %2.1lf%%";
	push @rrdcmd, "GPRINT:percent:MAX:  Max\\: %2.1lf%%\\l";

	push @rrdcmd, "COMMENT: ";
	push @rrdcmd, "AREA:yellow#FFFF00:50-60%";
	push @rrdcmd, "AREA:yorang#FFCC00:60-70%";
	push @rrdcmd, "AREA:oyello#FF9900:70-80%";
	push @rrdcmd, "AREA:orange#FF6600:80-90%";
	push @rrdcmd, "AREA:red#FF0000:90-100%";
	push @rrdcmd, "GPRINT:percent:LAST:    Now\\: %2.1lf%%";
	push @rrdcmd, "GPRINT:percent:AVERAGE:  Avg\\: %2.1lf%%\\n";

	push @rrdcmd, "COMMENT:\\n";
	push @rrdcmd, "COMMENT:Load Average\\n";
	push @rrdcmd, "COMMENT: ";
	push @rrdcmd, "LINE1:one#9900FF:1 minute";
	push @rrdcmd, "COMMENT:                                          ";
	push @rrdcmd, "GPRINT:one:MIN:Min\\: %2.1lf";
	push @rrdcmd, "GPRINT:one:MAX:   Max\\: %2.1lf";

	push @rrdcmd, "COMMENT:\\n";
	push @rrdcmd, "COMMENT: ";
	push @rrdcmd, "LINE1:five#FF00FF:5 minutes";
	push @rrdcmd, "COMMENT:                                         ";
	push @rrdcmd, "GPRINT:one:LAST:Now\\: %2.1lf";
	push @rrdcmd, "GPRINT:one:AVERAGE:   Avg\\: %2.1lf\\n";
	push @rrdcmd, "COMMENT: ";
	push @rrdcmd, "LINE1:fifteen#000000:15 minutes\\n";
	push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf) . "\\n";
	
	GenerateGraph($conf, $image, @rrdcmd);
};

sub GraphMemory($$$)
{
	my ($host, $descr, $conf) = @_;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $datasrc = "$datadir/$host/${descr}.rrd";
	my $image   = "$htmldir/$host/${descr}.png";

	return 0 if ( -e $image && ! $utility->file_check_writable($image) );
	$utility->file_check_readable($datasrc);

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my   @rrdcmd = $image;
	push @rrdcmd, "--start=-$seconds";

	push @rrdcmd, "--title=Memory Usage";
	push @rrdcmd, "-v Memory Usage";
	push @rrdcmd, "--lower-limit=0";
#	push @rrdcmd, "--upper-limit=120";
#	push @rrdcmd, "--rigid";
#	push @rrdcmd, "--units-exponent=3";
	push @rrdcmd, "--height=300";
	push @rrdcmd, "--width=500";
	push @rrdcmd, "--base=1024";
	push @rrdcmd, font_set($conf) if font_set($conf);

	push @rrdcmd, "DEF:mem1=$datasrc:swap:AVERAGE";
	push @rrdcmd, "DEF:mem2=$datasrc:swapavail:AVERAGE";
	push @rrdcmd, "DEF:mem3=$datasrc:total:AVERAGE";
	push @rrdcmd, "DEF:mem4=$datasrc:avail:AVERAGE";
	push @rrdcmd, "DEF:mem5=$datasrc:free:AVERAGE";
	push @rrdcmd, "DEF:mem6=$datasrc:shared:AVERAGE";
	push @rrdcmd, "DEF:mem7=$datasrc:buffer:AVERAGE";
	push @rrdcmd, "DEF:mem8=$datasrc:cached:AVERAGE";

	push @rrdcmd, "CDEF:swap_total=mem1,1000,\*";
	push @rrdcmd, "CDEF:swap_avail=mem2,1000,\*";
	push @rrdcmd, "CDEF:swap_used=swap_total,swap_avail,-";
	push @rrdcmd, "CDEF:mem_total=mem3,1000,\*";
	#push @rrdcmd, "CDEF:mem_total=mem3,1000,\*";
	push @rrdcmd, "CDEF:mem_avail=mem4,1000,\*";
	push @rrdcmd, "CDEF:mem_free=mem5,1000,\*";
	push @rrdcmd, "CDEF:mem_shared=mem6,1000,\*";
	push @rrdcmd, "CDEF:mem_buffer=mem7,1000,\*";
	push @rrdcmd, "CDEF:mem_cached=mem8,1000,\*";

	push @rrdcmd, "COMMENT:                     Min          Cur         Avg        Max\\n";

	push @rrdcmd, "AREA:mem_shared#00FFFF:Shared       ";        # aqua
	push @rrdcmd, "GPRINT:mem_shared:MIN: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_shared:LAST: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_shared:AVERAGE: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_shared:MAX: %3.1lf%S                                  \\j";
	#push @rrdcmd, "COMMENT:\\n";

	push @rrdcmd, "STACK:mem_buffer#3399FF:Buffer       ";       # light blue
	push @rrdcmd, "GPRINT:mem_buffer:MIN: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_buffer:LAST: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_buffer:AVERAGE: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_buffer:MAX: %3.1lf%S                                  \\j";
	#push @rrdcmd, "COMMENT:\\n";

	push @rrdcmd, "STACK:mem_cached#0000FF:Cached        ";       # blue
	push @rrdcmd, "GPRINT:mem_cached:MIN: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_cached:LAST: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_cached:AVERAGE: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_cached:MAX: %3.1lf%S                                  \\j";
	#push @rrdcmd, "COMMENT:\\n";
	
	push @rrdcmd, "STACK:mem_free#99FF00:Free          ";
	push @rrdcmd, "GPRINT:mem_free:MIN: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_free:LAST: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_free:AVERAGE: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_free:MAX: %3.1lf%S                                  \\j";
	#push @rrdcmd, "COMMENT:\\n";

	push @rrdcmd, "STACK:mem_avail#00CC00:Real Available";
	push @rrdcmd, "GPRINT:mem_avail:MIN: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_avail:LAST: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_avail:AVERAGE: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_avail:MAX: %3.1lf%S";
	push @rrdcmd, "GPRINT:mem_total:LAST:            Mem Total\\: %4.1lf%S\\j";
	#push @rrdcmd, "COMMENT:\\n";

	#push @rrdcmd, "LINE2:mem_total#0000FF:Real Total    ";
	#push @rrdcmd, "GPRINT:mem_total:LAST:       %3.1lf%s   ";
	#push @rrdcmd, "GPRINT:mem_total:MAX:        %3.1lf%s";
	#push @rrdcmd, "COMMENT:\\n";

	push @rrdcmd, "LINE2:swap_used#FF0000:Swap In Use\\:  ";
	push @rrdcmd, "GPRINT:swap_used:MIN: %3.1lf%S";
	push @rrdcmd, "GPRINT:swap_used:LAST: %3.1lf%S";
	push @rrdcmd, "GPRINT:swap_used:AVERAGE: %3.1lf%S";
	push @rrdcmd, "GPRINT:swap_used:MAX: %3.1lf%S";
	push @rrdcmd, "GPRINT:swap_total:LAST:           Swap Total\\:%4.1lf%S\\j";
	push @rrdcmd, "COMMENT:\\n";

	GenerateGraph($conf, $image, @rrdcmd);
};

sub GraphStorage($$$)
{
	my ($host, $polls, $conf) = @_;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my @color = format_colors( split(',', $conf->{'colors'}) );

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my $image   = "$htmldir/$host/disk_storage.png";

	return 0 if ( -e $image && ! $utility->file_check_writable($image) );

	print "GraphStorage: doing storage \n" if $debug;

	my   @rrdcmd = $image;
	push @rrdcmd, "--start=-$seconds";
	push @rrdcmd, "--upper-limit=100";
	push @rrdcmd, "--lower-limit=0";
	push @rrdcmd, "--vertical-label=Percent(%)";
	push @rrdcmd, "--title=Disk Capacity";
	push @rrdcmd, "COMMENT:\\s";
	push @rrdcmd, font_set($conf) if font_set($conf);

	my $j = 0;

	POLL: foreach my $poll (@$polls)          # Iterate through list of all polls
	{
		my ($type, $detail);
		until ( $type ) { ($type, $detail) = each %$poll };
		
		next POLL unless ($type eq "storage");
	
		my $mon = $detail->[0];
		my $descr = $detail->[1];

		my $file = "$datadir/$host/${descr}.rrd";
		$utility->file_check_readable($file);

		push @rrdcmd, "DEF:totalkbytes${j}=$file:total:AVERAGE";
		push @rrdcmd, "DEF:availkbytes${j}=$file:avail:AVERAGE";
		push @rrdcmd, "DEF:usedkbytes${j}=$file:used:AVERAGE";
		push @rrdcmd, "DEF:percent${j}=$file:percent:AVERAGE";

		push @rrdcmd, "CDEF:total${j}=totalkbytes${j},1024,\*";
		push @rrdcmd, "CDEF:avail${j}=availkbytes${j},1024,\*";
		push @rrdcmd, "CDEF:used${j}=usedkbytes${j},1024,\*";
 
		push @rrdcmd, "LINE2:percent${j}#$color[$j]: ";

		push @rrdcmd, "GPRINT:percent${j}:LAST:%3.0lf%%%s";
		push @rrdcmd, "GPRINT:used${j}:LAST:  Used\\:%4.0lf%s";
		push @rrdcmd, "GPRINT:avail${j}:LAST:  Free\\:%4.0lf%s";
		push @rrdcmd, "GPRINT:total${j}:LAST:  Total\\:%4.0lf%s";
		push @rrdcmd, "COMMENT:   $mon\\n";

		$j++;
	};

	push @rrdcmd, "COMMENT:\\s";
	push @rrdcmd, "COMMENT:                      Last updated " . formatted_timestamp($conf);

	GenerateGraph($conf, $image, @rrdcmd);
};

sub GraphDiskBytes($$$$)
{
	my ($host, $mon, $descr, $conf) = @_;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $datasrc = "$datadir/$host/${descr}.rrd";
	my $image   = "$htmldir/$host/${descr}.png";

	return 0 if ( -e $image && ! $utility->file_check_writable($image) );
	$utility->file_check_readable($datasrc);

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my   @rrdcmd = $image;
	push @rrdcmd, "--start=-$seconds";

	push @rrdcmd, "-v bytes per second";
	push @rrdcmd, "--title=Disk $mon Activity";
	push @rrdcmd, font_set($conf) if font_set($conf);

	push @rrdcmd, "DEF:rbits=$datasrc:read:AVERAGE";
	push @rrdcmd, "DEF:wbits=$datasrc:written:AVERAGE";
	push @rrdcmd, "DEF:rops=$datasrc:reads:AVERAGE";
	push @rrdcmd, "DEF:wops=$datasrc:writes:AVERAGE";

	push @rrdcmd, "CDEF:rbytes=rbits,8,\/";
	push @rrdcmd, "CDEF:wbytes=wbits,8,\/";
	push @rrdcmd, "CDEF:rundef=rops,0,GT,0,0,IF";
	push @rrdcmd, "CDEF:wundef=wops,0,GT,0,0,IF";

	push @rrdcmd, "COMMENT:                       ";
	push @rrdcmd, "COMMENT:Current      Average      Minimum      Maximum\\n";
	push @rrdcmd, "COMMENT:Disk Throughput (bps)\\n";
	push @rrdcmd, "COMMENT:         ";
	push @rrdcmd, "LINE1:rbytes#00FF00:   Read";
	push @rrdcmd, GetLegend("rbytes", $conf, 9, 0, 1, 0);

	push @rrdcmd, "COMMENT:\\n";
	push @rrdcmd, "COMMENT:         ";
	push @rrdcmd, "LINE2:wbytes#0000FF:Written";
	push @rrdcmd, GetLegend("wbytes", $conf, 9, 0, 1, 0);

	push @rrdcmd, "COMMENT:\\n";
	push @rrdcmd, "COMMENT:\\n";
	push @rrdcmd, "COMMENT:Disk Operations per second\\n";
	push @rrdcmd, "COMMENT:         ";
	push @rrdcmd, "LINE1:rundef#00FF00:  Reads";
	push @rrdcmd, GetLegend("rops", $conf, 9, 1, 1, 0);

	push @rrdcmd, "COMMENT:\\n";
	push @rrdcmd, "COMMENT:         ";
	push @rrdcmd, "LINE1:wundef#0000FF: Writes";
	push @rrdcmd, GetLegend("wops", $conf, 9, 1, 1, 0);

	GenerateGraph($conf, $image, @rrdcmd);
};

sub GraphNetwork($$$$)
{
	my ($host, $descr, $mon, $conf) = @_;

	if ($mon ne "all") {
		return GraphNetworkDevice($host, $descr, $mon, $conf);
	};

	my $datadir = $conf->{'datadir'};
	foreach ( $utility->get_dir_files("$datadir/$host") ) {
		if ( $_ =~ /net_(.*)\.rrd$/ ) {
			next if ( $1 eq "all");
			GraphNetworkDevice($host, "net_$1", $1, $conf);
		};
	};
};

sub GraphNetworkDevice($$$$)
{
	my ($host, $descr, $mon, $conf) = @_;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};
	my $bits    = $conf->{'bits'};

	my $datasrc = "$datadir/$host/${descr}.rrd";
	my $image   = "$htmldir/$host/${descr}.png";

	return 0 if ( -e $image && ! $utility->file_check_writable($image) );
	$utility->file_check_readable($datasrc);

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;

	my   @rrdcmd = $image;
	push @rrdcmd, "--start=-$seconds";
	if ( substr($descr,4) ne $mon ) {
		push @rrdcmd, "--title=Interface $descr ($mon)";
	} else {
		push @rrdcmd, "--title=Interface $descr";
	}
	push @rrdcmd, "--width=480";
	push @rrdcmd, "--imgformat=PNG";
	push @rrdcmd, font_set($conf) if font_set($conf);

	push @rrdcmd, "DEF:inoctets=$datasrc:in:AVERAGE";
	push @rrdcmd, "DEF:outoctets=$datasrc:out:AVERAGE";
	push @rrdcmd, "COMMENT:\\s";

	my $label;
	# An octet is a byte!
	if ( $bits )           # b is for bits
	{
		push @rrdcmd, "-v bits per second";
		push @rrdcmd, "CDEF:in=inoctets,8,\*";      # bits = octets * 8
		push @rrdcmd, "CDEF:out=outoctets,8,\*";
	}
	else                 # B is for bytes or octets
	{
		push @rrdcmd, "-v bytes per second";
		push @rrdcmd, "CDEF:in=inoctets";
		push @rrdcmd, "CDEF:out=outoctets";
	};

	if ( $conf->{'rrdtool_version'} == 2 )
	{
		push @rrdcmd, 'VDEF:ds0pct=in,95,PERCENT';
		push @rrdcmd, 'VDEF:ds1pct=out,95,PERCENT';
		push @rrdcmd, "COMMENT:              Min        Cur        Avg        Max       95th Percentile\\n";
	}  else {
		push @rrdcmd, "COMMENT:              Min        Cur        Avg        Max            \\n";
	}

	push @rrdcmd, "AREA:in#00FF00: in    ";
	push @rrdcmd, "GPRINT:in:MIN: %5.1lf %s ";
	push @rrdcmd, "GPRINT:in:LAST: %5.1lf %s ";
	push @rrdcmd, "GPRINT:in:AVERAGE: %5.1lf %s ";
	push @rrdcmd, "GPRINT:in:MAX: %5.1lf %s ";
	push @rrdcmd, "GPRINT:ds0pct: %5.2lf %s" if ( $conf->{'rrdtool_version'} == 2 );
	push @rrdcmd, "COMMENT:\\n";

	push @rrdcmd, "LINE1:out#0000FF: out   ";
	push @rrdcmd, "GPRINT:out:MIN: %5.1lf %s ";
	push @rrdcmd, "GPRINT:out:LAST: %5.1lf %s ";
	push @rrdcmd, "GPRINT:out:AVERAGE: %5.1lf %s ";
	push @rrdcmd, "GPRINT:out:MAX: %5.1lf %s ";
	push @rrdcmd, "GPRINT:ds1pct: %5.2lf %s " if ( $conf->{'rrdtool_version'} == 2 );
	push @rrdcmd, "COMMENT:\\n";
	
	GenerateGraph($conf, $image, @rrdcmd);
};

sub GraphMysql($$$$)
{
	my ($host, $mon, $descr, $conf) = @_;
	my @rrdcmd;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;
 
	print "GraphMysql being called for $host ($mon, $descr)<br>\n" if $debug;

	my $datasrc = "$datadir/$host/${descr}.rrd";
	$utility->file_check_readable($datasrc);

	my @color = format_colors( split(',', $conf->{'colors'}) );

	if ( $mon eq "queries" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMysql: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v queries / second";
		push @rrdcmd, "--title=Mysql usage";
		#push @rrdcmd, "--width=510";
		push @rrdcmd, "--width=480";
		push @rrdcmd, "--height=180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:mysql_queriessec:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:mysql_slowqueries:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:mysql_connections:AVERAGE";

		push @rrdcmd, "COMMENT:                   Min    Cur    Avg    Max\\n";

		push @rrdcmd, "AREA:a1#0000FF:Queries\\:     ";
		push @rrdcmd, "GPRINT:a1:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a1:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a1:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a1:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "LINE1:a3#FF0000:Slow queries\\:";
		push @rrdcmd, "GPRINT:a3:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a3:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a3:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a3:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "AREA:a2#FFFF00:Connections\\: ";
		push @rrdcmd, "GPRINT:a2:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a2:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a2:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a2:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT: (Con/sec)     " . formatted_timestamp($conf) . "\r\\n";
		
		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif( $mon eq "type_queries" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMysql: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v queries / second";
		push @rrdcmd, "--title=MySQL Query Types";
		push @rrdcmd, "--width=480";
		push @rrdcmd, "--height=180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:mysql_select:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:mysql_insert:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:mysql_delete:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:mysql_update:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:mysql_changedb:AVERAGE";
		
		push @rrdcmd, "COMMENT:Query Type     Min     Cur    Avg     Max\\n";

		push @rrdcmd, "AREA:a1#0000FF:Select\\:   ";
		push @rrdcmd, "GPRINT:a1:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a1:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a1:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a1:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "STACK:a2#00FF00:Insert\\:   ";
		push @rrdcmd, "GPRINT:a2:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a2:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a2:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a2:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "STACK:a4#FFFF00:Update\\:   ";
		push @rrdcmd, "GPRINT:a4:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a4:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a4:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a4:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:a3#FF0000:Delete\\:   ";
		push @rrdcmd, "GPRINT:a3:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a3:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a3:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a3:MAX: %4.1lf\r";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "STACK:a5#FF00FF:Change DB\\:";
		push @rrdcmd, "GPRINT:a5:MIN: %3.1lf\r";
		push @rrdcmd, "GPRINT:a5:LAST: %4.1lf\r";
		push @rrdcmd, "GPRINT:a5:AVERAGE: %3.1lf\r";
		push @rrdcmd, "GPRINT:a5:MAX: %4.1lf";
		push @rrdcmd, "COMMENT:                  " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif( $mon eq "combined" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMysql: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v queries / second";
		push @rrdcmd, "--title=MySQL Activity";
		push @rrdcmd, "--width=480";
		push @rrdcmd, "--height=180";
		push @rrdcmd, "--units-exponent=1";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:my1:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:my2:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:my3:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:my4:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:my5:AVERAGE";
		push @rrdcmd, "DEF:a6=$datasrc:my6:AVERAGE";
		push @rrdcmd, "DEF:a7=$datasrc:my7:AVERAGE";
		push @rrdcmd, "DEF:a8=$datasrc:my8:AVERAGE";

		push @rrdcmd, "COMMENT:                 Min     Cur    Avg     Max";
		push @rrdcmd, "COMMENT:      Query Type     Min    Cur    Avg    Max\\n";

		push @rrdcmd, "LINE2:a8#9900FF:Queries\\:   ";
		push @rrdcmd, "GPRINT:a8:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a8:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a8:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a8:MAX: %4.1lf   ";

		push @rrdcmd, "AREA:a1#0000FF:Select\\: ";
		push @rrdcmd, "GPRINT:a1:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a1:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a1:MAX: %4.1lf";
		push @rrdcmd, "COMMENT:\\j";

		push @rrdcmd, "STACK:a6#FFCC00:Connects\\:  ";
		push @rrdcmd, "GPRINT:a6:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a6:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a6:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a6:MAX: %4.1lf  ";

		push @rrdcmd, "STACK:a2#99FF00:Insert\\:  ";
		push @rrdcmd, "GPRINT:a2:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a2:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a2:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a2:MAX: %4.1lf";
		push @rrdcmd, "COMMENT:\\j";

		push @rrdcmd, "STACK:a7#FF0000:Slow Query\\:";
		push @rrdcmd, "GPRINT:a7:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a7:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a7:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a7:MAX: %4.1lf  ";

		push @rrdcmd, "STACK:a4#00CC00:Update\\:  ";
		push @rrdcmd, "GPRINT:a4:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a4:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a4:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a4:MAX: %4.1lf";
		push @rrdcmd, "COMMENT:\\j";

		push @rrdcmd, "COMMENT:                                              ";
		push @rrdcmd, "STACK:a3#FFFF00:Delete\\:  ";
		push @rrdcmd, "GPRINT:a3:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a3:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a3:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a3:MAX: %4.1lf";
		push @rrdcmd, "COMMENT:\\j";
		#push @rrdcmd, "COMMENT:                                             \\j";

		push @rrdcmd, "COMMENT:" . formatted_timestamp($conf) . "                  ";
		push @rrdcmd, "STACK:a5#FF00FF:ChangeDB\\:";
		push @rrdcmd, "GPRINT:a5:MIN: %4.1lf";
		push @rrdcmd, "GPRINT:a5:LAST: %4.1lf";
		push @rrdcmd, "GPRINT:a5:AVERAGE: %4.1lf";
		push @rrdcmd, "GPRINT:a5:MAX: %4.1lf";
		push @rrdcmd, "COMMENT:\\j";

		GenerateGraph($conf, $image, @rrdcmd);
	}

}

sub GraphMbmon($$$$)
{
	my ($host, $mon, $descr, $conf) = @_;
	my @rrdcmd;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;
 
	print "GraphMbmon being called for $host ($mon, $descr)<br>\n" if $debug;

	my $datasrc = "$datadir/$host/${descr}.rrd";
	$utility->file_check_readable($datasrc);

	if ( $mon eq "temperature" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMbmon: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Temprature (C)";
		push @rrdcmd, "--title=Temperature monitor";
#		push @rrdcmd, "--width=480";
#		push @rrdcmd, "-h 180";

		push @rrdcmd, "DEF:a1=$datasrc:mbmon_temp0:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:mbmon_temp1:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:mbmon_temp2:AVERAGE";

		push @rrdcmd, "LINE2:a1#0000FF: MB Temp ";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE2:a2#FFFF00: Temp1   ";
		push @rrdcmd, "GPRINT:a2:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE2:a3#FF0000: CPU Temp";
		push @rrdcmd, "GPRINT:a3:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a3:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a3:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a3:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif( $mon eq "voltage" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMbmon: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Voltage";
		push @rrdcmd, "--title=Voltage monitor";
#		push @rrdcmd, "--width=480";
#		push @rrdcmd, "-h 180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:mbmon_vc0:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:mbmon_vc1:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:mbmon_v33:AVERAGE";
		push @rrdcmd, "DEF:a4=$datasrc:mbmon_v50p:AVERAGE";
		push @rrdcmd, "DEF:a5=$datasrc:mbmon_v12p:AVERAGE";
		#push @rrdcmd, "DEF:a6=$datasrc:mbmon_v12n:GAUGE";
		#push @rrdcmd, "DEF:a7=$datasrc:mbmon_v50n:GAUGE";
		
		push @rrdcmd, "LINE1:a1#0000FF: Core voltage 0";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.2lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.2lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.2lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.2lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE1:a2#FF00FF: Core voltage 1";
		push @rrdcmd, "GPRINT:a2:MIN:Min\\: %3.2lf";
		push @rrdcmd, "GPRINT:a2:MAX:Max\\: %3.2lf";
		push @rrdcmd, "GPRINT:a2:LAST:Current\\: %3.2lf";
		push @rrdcmd, "GPRINT:a2:AVERAGE:Average\\: %3.2lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "LINE1:a3#FFFF00: Voltage 3.33  ";
		push @rrdcmd, "GPRINT:a3:MIN:Min\\: %3.2lf";
		push @rrdcmd, "GPRINT:a3:MAX:Max\\: %3.2lf";
		push @rrdcmd, "GPRINT:a3:LAST:Current\\: %3.2lf";
		push @rrdcmd, "GPRINT:a3:AVERAGE:Average\\: %3.2lf";
		push @rrdcmd, "COMMENT:\\n";

		push @rrdcmd, "LINE1:a4#00FF00: Voltage 5.0   ";
		push @rrdcmd, "GPRINT:a4:MIN:Min\\: %3.2lf";
		push @rrdcmd, "GPRINT:a4:MAX:Max\\: %3.2lf";
		push @rrdcmd, "GPRINT:a4:LAST:Current\\: %3.2lf";
		push @rrdcmd, "GPRINT:a4:AVERAGE:Average\\: %3.2lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE1:a5#FF0000: Voltage 12    ";
		push @rrdcmd, "GPRINT:a5:MIN:Min\\: %3.2lf";
		push @rrdcmd, "GPRINT:a5:MAX:Max\\: %3.0lf";
		push @rrdcmd, "GPRINT:a5:LAST:Current\\: %3.2lf";
		push @rrdcmd, "GPRINT:a5:AVERAGE:Average\\: %3.2lf";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif ( $mon eq "fanspeed" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphMbmon: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Fanspeed (RPM)";
		push @rrdcmd, "--title=Fanspeed monitor";
#		push @rrdcmd, "--width=480";
#		push @rrdcmd, "-h 180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:mbmon_fan0:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:mbmon_fan1:AVERAGE";
		push @rrdcmd, "DEF:a3=$datasrc:mbmon_fan2:AVERAGE";

		push @rrdcmd, "LINE2:a1#0000FF: Fan 0 ";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.0lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.0lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.0lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.0lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE2:a2#FFFF00: Fan 1 ";
		push @rrdcmd, "GPRINT:a2:MIN:Min\\: %3.0lf";
		push @rrdcmd, "GPRINT:a2:MAX:Max\\: %3.0lf";
		push @rrdcmd, "GPRINT:a2:LAST:Current\\: %3.0lf";
		push @rrdcmd, "GPRINT:a2:AVERAGE:Average\\: %3.0lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE2:a3#FF0000: Fan 2 ";
		push @rrdcmd, "GPRINT:a3:MIN:Min\\: %3.0lf";
		push @rrdcmd, "GPRINT:a3:MAX:Max\\: %3.0lf";
		push @rrdcmd, "GPRINT:a3:LAST:Current\\: %3.0lf";
		push @rrdcmd, "GPRINT:a3:AVERAGE:Average\\: %3.0lf";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}
}

sub GraphApache($$$$)
{
	my ($host, $mon, $descr, $conf) = @_;
	my @rrdcmd;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $seconds = 86400;
	   $seconds = $days * 86400 if $days;
 
	print "GraphApache being called for $host ($mon, $descr)<br>\n" if $debug;

	my $datasrc = "$datadir/$host/${descr}.rrd";
	$utility->file_check_readable($datasrc);

	if ( $mon eq "workers" || $mon eq "combined" )
	{
		my $image = "$htmldir/$host/${descr}.png";
		   $image = "$htmldir/$host/apache_workers.png" if ($mon eq "combined");

		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphApache workers: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Apache servers";
		push @rrdcmd, "--title=Apache Child Status";
#		push @rrdcmd, "--width=480";
#		push @rrdcmd, "-h 180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		if ( $mon eq "combined" ) {
			push @rrdcmd, "DEF:a1=$datasrc:workers_busy:AVERAGE";
			push @rrdcmd, "DEF:a2=$datasrc:workers_idle:AVERAGE";
		} else {
			push @rrdcmd, "DEF:a1=$datasrc:apache_workers_busy:AVERAGE";
			push @rrdcmd, "DEF:a2=$datasrc:apache_workers_idle:AVERAGE";
		}

		push @rrdcmd, "AREA:a1#0000FF: Busy Servers ";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "STACK:a2#FFFF00: Idle Servers ";
		push @rrdcmd, "GPRINT:a2:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}

	if( $mon eq "bytes_per_req" || $mon eq "combined" )
	{
		my $image = "$htmldir/$host/${descr}.png";
		   $image = "$htmldir/$host/apache_bytes_per_req.png" if ($mon eq "combined");
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphApache bytes_per_req: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Apache Bytes";
		push @rrdcmd, "--title=Apache Bytes per request";
		push @rrdcmd, "--width=480";
#		push @rrdcmd, "-h 180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		if ($mon eq "combined") {
			push @rrdcmd, "DEF:a1=$datasrc:bytes_per_req:AVERAGE";
		} else {
			push @rrdcmd, "DEF:a1=$datasrc:apache_per_req:AVERAGE";
		}
		
		push @rrdcmd, "AREA:a1#0000FF: Bytes per request ";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}

	if( $mon eq "req_per_sec" || $mon eq "combined" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		   $image = "$htmldir/$host/apache_req_per_sec.png" if ($mon eq "combined");
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphApache req_per_sec: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Apache Requests";
		push @rrdcmd, "--title=Apache Requests per Second";
#		push @rrdcmd, "--width=480";
#		push @rrdcmd, "-h 180";
		push @rrdcmd, "--units-exponent=1";
		push @rrdcmd, font_set($conf) if font_set($conf);

		if ( $mon eq "combined") {
			push @rrdcmd, "DEF:a1=$datasrc:req_per_sec:AVERAGE";
		} else {
			push @rrdcmd, "DEF:a1=$datasrc:apache_req_per_sec:AVERAGE";
		}

		push @rrdcmd, "AREA:a1#0000FF: Request per sec ";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.1lf";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}
}

sub GraphPF($$$$)
{
	my ($host, $mon, $descr, $conf) = @_;
	my @rrdcmd;

	my $days    = $conf->{'days'};
	my $debug   = $conf->{'debug'};
	my $htmldir = $conf->{'htmldir'};
	my $datadir = $conf->{'datadir'};

	my $seconds = 86400;
	$seconds = $days * 86400 if $days;
 
	print "GraphPF being called for $host ($mon, $descr)<br>\n" if $debug;

	my $datasrc = "$datadir/$host/${descr}.rrd";
	$utility->file_check_readable($datasrc);

	if ( $mon eq "states" )
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphPF states: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v states";
#	    push @rrdcmd, "--width=600";
		push @rrdcmd, font_set($conf) if font_set($conf);
        push @rrdcmd, "--title=PF active states";

        push @rrdcmd, "DEF:s=$datasrc:states_entries:AVERAGE";
        push @rrdcmd, "DEF:si=$datasrc:states_inserts:AVERAGE";
        push @rrdcmd, "DEF:sr=$datasrc:states_removals:AVERAGE";
        push @rrdcmd, "DEF:ss=$datasrc:states_searches:AVERAGE";
        push @rrdcmd, "CDEF:msr=-1,sr,*";
        push @rrdcmd, "COMMENT:                      min     avg     max        last\\n";
        push @rrdcmd, "AREA:s#00FF00:state entries  ";
        push @rrdcmd, "GPRINT:s:MIN:%6.2lf";
        push @rrdcmd, "GPRINT:s:AVERAGE:%6.2lf";
        push @rrdcmd, "GPRINT:s:MAX:%6.2lf";
        push @rrdcmd, "GPRINT:s:LAST:%6.2lf\\n";
        push @rrdcmd, "LINE1:msr#FF0000:state removals ";
        push @rrdcmd, "GPRINT:sr:MIN:%6.2lf";
        push @rrdcmd, "GPRINT:sr:AVERAGE:%6.2lf";
        push @rrdcmd, "GPRINT:sr:MAX:%6.2lf";
        push @rrdcmd, "GPRINT:sr:LAST:%6.2lf\\n";
        push @rrdcmd, "LINE1:si#0000FF:state inserts  ";
        push @rrdcmd, "GPRINT:si:MIN:%6.2lf";
        push @rrdcmd, "GPRINT:si:AVERAGE:%6.2lf";
        push @rrdcmd, "GPRINT:si:MAX:%6.2lf";
        push @rrdcmd, "GPRINT:si:LAST:%6.2lf\\n";
        push @rrdcmd, "LINE2:ss#00FFFF:state searches";
        push @rrdcmd, "GPRINT:ss:MIN: %6.2lf";
        push @rrdcmd, "GPRINT:ss:AVERAGE:%6.2lf";
        push @rrdcmd, "GPRINT:ss:MAX:%6.2lf";
        push @rrdcmd, "GPRINT:ss:LAST:%6.2lf\\n";
		
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif( $mon eq "ipbytes")
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphPF ipbytes: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Traffic through firewall";
		push @rrdcmd, "--title=Traffic through firewall";
		push @rrdcmd, "--width=480";
		push @rrdcmd, font_set($conf) if font_set($conf);

        push @rrdcmd, "DEF:A=$datasrc:bytes_v4_in:AVERAGE";
        push @rrdcmd, "DEF:B=$datasrc:bytes_v4_out:AVERAGE";
        push @rrdcmd, "DEF:C=$datasrc:bytes_v6_in:AVERAGE";
        push @rrdcmd, "DEF:D=$datasrc:bytes_v6_out:AVERAGE";

        push @rrdcmd, "CDEF:inb=A,8,*";
        push @rrdcmd, "CDEF:outb=B,8,*";
        push @rrdcmd, "CDEF:nodata=A,UN,0,*";

        push @rrdcmd, "COMMENT:                min          avg          max          last\\n";
        push @rrdcmd, "LINE1:nodata#FF0000";
        push @rrdcmd, "AREA:inb#00FF00:incoming";
        push @rrdcmd, "GPRINT:inb:MIN:%6.2lf %sbps";
        push @rrdcmd, "GPRINT:inb:AVERAGE:%6.2lf %sbps";
        push @rrdcmd, "GPRINT:inb:MAX:%6.2lf %sbps";
        push @rrdcmd, "GPRINT:inb:LAST:%6.2lf %sbps\\n";
        push @rrdcmd, "LINE1:outb#00FFFF:outgoing";
        push @rrdcmd, "GPRINT:outb:MIN:%6.2lf %sbps";
        push @rrdcmd, "GPRINT:outb:AVERAGE:%6.2lf %sbps";
        push @rrdcmd, "GPRINT:outb:MAX:%6.2lf %sbps";
        push @rrdcmd, "GPRINT:outb:LAST:%6.2lf %sbps\\n";

		GenerateGraph($conf, $image, @rrdcmd);
	}
	elsif( $mon eq "firewall")
	{
		my $image   = "$htmldir/$host/${descr}.png";
		return 0 if ( -e $image && ! $utility->file_check_writable($image) );

		print "GraphPF: passed checks.<br>\n" if ($debug);
			
		     @rrdcmd = $image;
		push @rrdcmd, "--start=-$seconds";
		push @rrdcmd, "-v Firewall stats";
		push @rrdcmd, "--title=PF Firewall stats";
		push @rrdcmd, "--width=500";
		push @rrdcmd, "--height=180";
		push @rrdcmd, font_set($conf) if font_set($conf);

		push @rrdcmd, "DEF:a1=$datasrc:passed:AVERAGE";
		push @rrdcmd, "DEF:a2=$datasrc:blocked:AVERAGE";
		

		push @rrdcmd, "AREA:a1#0000FF:Passed packets ";
		push @rrdcmd, "GPRINT:a1:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a1:AVERAGE:Average\\: %3.1lf ";
		push @rrdcmd, "COMMENT:\\n";
		
		push @rrdcmd, "LINE1:a2#FF0000:Blocked packets";
		push @rrdcmd, "GPRINT:a2:MIN:Min\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:MAX:Max\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:LAST:Current\\: %3.1lf";
		push @rrdcmd, "GPRINT:a2:AVERAGE:Average\\: %3.1lf ";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:\\n";
		push @rrdcmd, "COMMENT:                         Last updated " . formatted_timestamp($conf);

		GenerateGraph($conf, $image, @rrdcmd);		
	}
}
1;
__END__


=head1 COPYRIGHT

Copyright (c) 2003-2005, The Network People, Inc. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=cut

