#!/usr/bin/perl
use strict;

#
# $Id: HTML.pm 413 2006-07-08 05:15:52Z matt $
#

package RRDutil::HTML;

use CGI qw(:standard);
use CGI::Carp qw( fatalsToBrowser );   # use Carp;
use POSIX qw(uname);
use vars qw($VERSION);

$VERSION = '1.6';

#	$rrdutil->get_requested_graphs
#	$rrdutil->check_html_dir

use lib "../";
use RRDutil; my $rrdutil = RRDutil->new();

=head1 NAME

RRDutil::HTML - Perl module for rrdutil.pl


=head1 SYNOPSIS

RRDutil::HTML


=head1 DESCRIPTION

=cut


sub new
{
    my ($class, $name) = @_;
    my $self = { name => $name };
    bless ($self, $class);
    return $self;
};

sub graphs($$$$)
{
	my ($self, $hosts, $types_selected, $conf, $monitors) = @_;

	print '<!-- BEGIN HTML GRAPHS --><br></br><div align="center">';

	my $graphs = 0;
	my $url        = $conf->{'url'};

	foreach my $host ( @$hosts )
	{
		if ( param('load' ) ) 
		{ 
			foreach my $poll ( $rrdutil->get_polls($host, "load", $monitors) )
			{
				print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				print "<IMG SRC=$url/$host/${poll}_detail.png><br></br>";
				$graphs++;
			};
		};

		if ( param('disk' ) ) 
		{ 
			print "<IMG SRC=$url/$host/disk_storage.png><br></br>";

			my @polls = $rrdutil->get_polls($host, "disk", $monitors);
			foreach my $poll (@polls) 
			{
				print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				$graphs++;
			};

		};

		if ( param('net') ) 
		{ 
			my @polls = $rrdutil->get_polls($host, "net", $monitors);
			foreach my $poll (@polls) 
			{
				if ( $poll eq "net_all" ) {
					require RRDutil::Utility;
					my $utility = new RRDutil::Utility;
					foreach ( $utility->get_dir_files("$conf->{'datadir'}/$host") ) {
						if ( $_ =~ /net_(.*)\.rrd$/ ) {
							next if ( $1 eq "all");
							print "<IMG SRC=$url/$host/net_$1.png><br></br>";
						};
					};
				} else {
					print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				};
				$graphs++;
			};
		};
	
		if ( param('memory') )
		{ 
			my @polls = $rrdutil->get_polls($host, "memory", $monitors);
			foreach my $poll (@polls) 
			{
				print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				$graphs++;
			};
		};
	
		if ( param('www') ) 
		{
			my @polls = $rrdutil->get_polls($host, "www", $monitors);
	
			my $i = 0;
	
			foreach my $poll (@polls) 
			{
				if ( $poll ne "webhits" ) 
				{
					print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				} 
				else 
				{
					unless ($i) 
					{
						print "<IMG SRC=$url/$host/${poll}.png><br></br>";
						$graphs++;
					};
					$i++;
				};
			};
		};
	
		if ( param('mail') ) 
		{
			foreach my $poll ( $rrdutil->get_polls($host, "mail", $monitors) )
			{
				print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				print "<IMG SRC=$url/$host/${poll}-2.png><br></br>" if ($poll eq "mail_rbl_blocks");
				$graphs++;
			};
		};

		if ( param('mysql') )
		{
			foreach my $poll ( $rrdutil->get_polls($host, "mysql", $monitors) )
			{
				print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				$graphs++;
			};
		};

		if ( param('mbmon') )
		{
			foreach my $poll ( $rrdutil->get_polls($host, "mbmon", $monitors) )
			{
				print "<IMG SRC=\"$url/$host/${poll}.png\"><br></br>";
				$graphs++;
			};
		};

		if ( param('apache') )
		{
			foreach my $poll ( $rrdutil->get_polls($host, "apache", $monitors) )
			{
				if ( $poll eq "apache" ) {
					print "<IMG SRC=$url/$host/apache_bytes_per_req.png><br></br>";
					print "<IMG SRC=$url/$host/apache_req_per_sec.png><br></br>";
					print "<IMG SRC=$url/$host/apache_workers.png><br></br>";
					$graphs++;
				} else {
					print "<IMG SRC=$url/$host/${poll}.png><br></br>";
					$graphs++;
				}
			};
		};
		
		if ( param('pf') )
		{
			foreach my $poll ( $rrdutil->get_polls($host, "pf", $monitors) )
			{
				print "<IMG SRC=$url/$host/${poll}.png><br></br>";
				$graphs++;
			};
		};
	};
	
	unless ($graphs) 
	{
		print '<font color="red">Select at least one graph, and click submit.</font></div><!-- END HTML GRAPHS --><br></br>';
	}
	else
	{
		print "</div><!-- END HTML GRAPHS --><br></br>";
	};
}

sub request($$;$$)
{
	my ($self, $hostlist, $avail, $refresh, $allchecked) = @_;

	my $font   = '<font face="Arial, Helvetica, sans-serif">';
	my $font2  = '<font face="Arial, Helvetica, sans-serif" size="+2">';
	my $font_1 = '<font face="Arial, Helvetica, sans-serif" size="-1">';
	my $font_2 = '<font face="Arial, Helvetica, sans-serif" size="-2">';

	# load disk memory net mail www storage mysql

	my ($net, $disk, $www, $cpu, $mysql, $memory, $mail, $next, $mbmon, $apache , $pf);

	if ( param('net')    ) { $net    = "checked" } else { $net = "/" };
	if ( param('disk')   ) { $disk   = "checked" } else { $disk = "/" };
	if ( param('www')    ) { $www    = "checked" } else { $www = "/" };
	if ( param('load')   ) { $cpu    = "checked" } else { $cpu = "/" };
	if ( param('mysql')  ) { $mysql  = "checked" } else { $mysql = "/" };
	if ( param('memory') ) { $memory = "checked" } else { $memory = "/" };
	if ( param('mail')   ) { $mail   = "checked" } else { $mail = "/" };
	if ( param('mbmon')  ) { $mbmon  = "checked" } else { $mbmon = "/" };
	if ( param('apache') ) { $apache = "checked" } else { $apache = "/" };
	if ( param('pf')   )   { $pf     = "checked" } else { $pf = "/" };

	my @list1 = $font_1 . 'Graph</font>';
	if ( $avail->{'net'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list1, $font_1 . '<input type="checkbox" name="net" value="on" checked '.$net.'> Network</font>';
		}
		else
		{
			push @list1, $font_1 . '<input type="checkbox" name="net" value="on" '.$net.'> Network</font>';
		}
	};

	if ( $avail->{'disk'} or $avail->{'storage'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list1, $font_1 . '<input type="checkbox" name="disk" value="on" checked '.$disk.'> Disk</font>';
		}
		else
		{
			push @list1, $font_1 . '<input type="checkbox" name="disk" value="on" '.$disk.'> Disk</font>';
		}
	};

	if ( $avail->{'www'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list1, $font_1 . '<input type="checkbox" name="www" value="on" checked '.$www.'> Web Hits</font>';
		}
		else
		{
			push @list1, $font_1 . '<input type="checkbox" name="www" value="on" '.$www.'> Web Hits</font>';
		}
	};

	if ( $avail->{'mysql'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list1, $font_1 . '<input type="checkbox" name="mysql" value="on" checked '.$mysql.'> Mysql </font>';
		}
		else
		{
			push @list1, $font_1 . '<input type="checkbox" name="mysql" value="on" '.$mysql.'> Mysql </font>';
		}
	};

	if ( $avail->{'mbmon'} ) 
	{
		if ( $allchecked == 1 )
		{
			push @list1, $font_1 . '<input type="checkbox" name="mbmon" value="on" checked '.$mbmon.'> Mbmon </font>';
		}
		else
		{
			push @list1, $font_1 . '<input type="checkbox" name="mbmon" value="on" '.$mbmon.'> Mbmon </font>';
		}
	};

	my @list2 = $font_1 . "Types</font>";
	if ( $avail->{'cpu'} or $avail->{'load'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list2, $font_1 . '<input type="checkbox" name="load" value="on" checked '.$cpu.'> CPU</font>' ;
		}
		else
		{
			push @list2, $font_1 . '<input type="checkbox" name="load" value="on" '.$cpu.'> CPU</font>' ;
		}
	};

	if ( $avail->{'memory'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list2, $font_1 . '<input type="checkbox" name="memory" value="on" checked '.$memory.'> Memory</font>';
		}
		else
		{
			push @list2, $font_1 . '<input type="checkbox" name="memory" value="on" '.$memory.'> Memory</font>';
		}
	};

	if ( $avail->{'mail'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list2, $font_1 . '<input type="checkbox" name="mail" value="on" checked '.$mail.'> Mail</font>';    
		}
		else
		{
			push @list2, $font_1 . '<input type="checkbox" name="mail" value="on" '.$mail.'> Mail</font>';
		}
	};

	if ( $avail->{'apache'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list2, $font_1 . '<input type="checkbox" name="apache" value="on" checked '.$apache.'> Apache</font>';        
		}
		else
		{
			push @list2, $font_1 . '<input type="checkbox" name="apache" value="on" '.$apache.'> Apache</font>';        
		}
	};
	
	if ( $avail->{'pf'} ) 
	{ 
		if ( $allchecked == 1 )
		{
			push @list2, $font_1 . '<input type="checkbox" name="pf" value="on" checked '.$pf.'> PF firewall</font>';        
		}
		else
		{
			push @list2, $font_1 . '<input type="checkbox" name="pf" value="on" '.$pf.'> PF firewall</font>';        
		}
	};
	

	my @daylist = qw( 0.5 1 2 3 7 30 60 90 180 365 );

	use Sys::Hostname;
	my $hostname = hostname();

	my $js = js_timer($refresh);

	print start_html(-title => "RRDutil Statistics - $hostname",
			-author  => "matt\@tnpi.biz",
# this doesn't work as it "forgets" the form settings
#			-head    => meta( {-http_equiv=>'REFRESH',-content=>'60;'} ),
			-meta    => { 'keywords'=>'network monitoring rrdtool rrdutil snmp management',
					'copyright'=>'copyright 2005 The Network People, Inc', },
			-BGCOLOR => 'white', 
			-script=>$js,
			-onLoad=>'timer()' ),
		table({-border=>0, -width=>"95%", -bgcolor=>'black'},
			Tr({-align=>'center', -valign=>'middle'},
				td({-height=>'100'},
					table({-border=>0, -width=>"100%", -bgcolor=>'#CCCCCC'},
						Tr({-align=>'left'}, 
							td({-width=>'25', height=>'25'}, '&nbsp;'), 
							td({-align=>'left'}, $font_1 . 
								'<a href="http://www.tnpi.biz/internet/manage/rrdutil/">RRDutil</a> Statistics for '.$hostname.'</font>' ),
							td({-align=>'right'}, $font_2. 
								'<a href="http://www.tnpi.biz/">The Network People, Inc.</a></font><br></br>' ),
							td({-width=>'25', -height=>'5'}, '&nbsp')
						),
#						Tr({-align=>'left', -bgcolor=>'green', -height=>'5'}, 
#							td( '' )
#						),
					),
					start_form(-name=>'rrdform'),
					table({-border=>0, -width=>"100%", -bgcolor=>'#999999'},
						Tr({-align=>'left'},
						[
## START
							td([ @list1 ]),
							td([ @list2 ]),
## END
						] ),
					),
					table({-border=>0, -width=>"100%", -bgcolor=>'#CCCCCC'},
						Tr({-align=>'left'},
							td( [ submit,
								$font_1 . 'Days: </font>' . 
									popup_menu(-onChange=>'submit()', -name=>'days', -values=>[@daylist], -default=>'1'), 
								$font_1 . 'Host: </font>' . 
									popup_menu(-onChange=>'submit()', -name=>'hostname', -values=>[@$hostlist], -default=>'localhost'),
									$font_1 . 'refresh: '.$refresh.' min<input type="text" value="" name="counter" size="5"></font>'
							] ),
						),
					),
					end_form,
				)
			) 
		);
	
	print "<!-- END HTML REQUEST FORM--><br></br>";
};




sub js_timer(;$)
{
	my ($minutes) = @_;

	unless ($minutes) { $minutes = 5; };

	my $foo = '

var min = "00"
var sec = "0"

function timer(){

	if ((min < 10) && (min != "00")){
		dismin = "0" + min
	}
	else{
		dismin = min
	}

	dissec = (sec < 10) ? sec = "0" + sec : sec
	document.rrdform.counter.value = dismin + ":" + dissec

	if (sec < 59){
		sec++
	}
	else{
		sec = "0"
		min++
		if (min >= '. $minutes . '){
			document.rrdform.submit()
		}
	}

	window.setTimeout("timer()",1000) 
}

';

	return $foo;
};

sub footer($)
{
	my ($self, $version) = @_;

	print "<!-- BEGIN HTML FOOTER --><br></br>";

	my $font_2 = '<font face="Arial, Helvetica, sans-serif" size="-2">';

	print '<center>', $font_2, '<a href="http://www.tnpi.biz/internet/manage/rrdutil/">RRDutil</a> v', $version, ' - &copy; <a href="http://www.tnpi.biz/">The Network People, Inc.</a> 1999-2005 </font></center><!-- END HTML FOOTER --></body></html>';
};



1;
__END__


Copyright (c) 2003-2005, The Network People, Inc. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
